------------------------------------------------------------------------
-- Copyright Notice:
-- (c) Copyright 2004 - 2008, National Optical Astronomy Observatory,
-- which is operated for the National Science Foundation by the
-- Association of Universities for Research in Astronomy under
-- Cooperative Agreement No. AST-0132798.
------------------------------------------------------------------------
-- PROJECT      :   MONSOON-TORRENT
-- BOARD        :   LOCAL CONTROL BOARD
-- DEVICE ID    :   VIRTEX 5LX
-- PARENT MODULE:   CFG_SERVICES
-- MODULE       :   REGISTER CONTROL
-- VERSION      :  2.20
------------------------------------------------------------------------
-- FILE NAME    :   CFG_RegisterControl_Ver220.vhd
-- AUTHOR         :   Peter Moore
-- DATE               :   03/05/2012
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 10.1.03
------------------------------------------------------------------------
-- DESCRIPTION  :  Registers to hold CFG Services module control parameters
--                         :  writen by the PAN or other modules across the WB Bus.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- REGISTER STRUCTURES:
-- eepRdCmdReg    at address 0x0020
-- eepWrtCmdReg    at address 0x0021
--
-- Temperature selection for DHE tempertaure servo is:
--  DheTempSensorSlct_reg 
--                  Value                                   Sensor Selected for servo reference
--                  "000"                                   fpgaTemp (From system monitor)
--                  "001"                                   LCB_temperature0  (From CFG module)
--                  "010"                                   LCB_temperature1  (From CFG module)
--                  "011"                                   AFE0_temperature  (From CFG module)
--                  "100"                                  AFE0_temperature1  (From CFG module)
--                  "101"                                   AFE1_temperature0  (From CFG module)
--                  "110"                                   AFE1_temperature1  (From CFG module)
--                  "111"                                    Manual seting from PAN (for testing purposes)

------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	     AUTHOR  VERSION  MODIFICATIONS
-- 06/18/2008  PCM     1.00    INITIAL VERSION
-- 01/26/2010   PCM     1.01    Added register set for the sequencer.
-- 02/04/2010   PCM     1.02   Minor edit
-- 02/18/2010    PCM     1.03   Removed clock control signals - Clock control now goes to a seperate module. Changed eepromCmdReg to be 17 bits.
-- 06/20/2010   PCM      1.05  Added mcbControl register for Nick.
-- 12/14/2010    PCM     2.02   Added shutter, shutter status and preflash polarity switches.
-- 02/27/2012   pcm     2.11  Added pause control to control shutter and preflash signals from sequencer  
-- 03/05/2012   PCM     2.20   Version value only .. This allows a firmware recovery point to be set at system version 2.11 
------------------------------------------------------------------------
-- PORT DEFINITIONS :

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity CFG_RegisterControl_V220 is
	Port (
-- WishBone bus signals
		WbSysClk 				: in   std_logic;
		WbReset 					: in   std_logic;
		WbWriteEnable 			: in   std_logic;
		WbAddress 				: in   std_logic_vector(15 downto 0);
		WbDataIn 				: in   std_logic_vector(31 downto 0);
		WbWrtAck 				: out  std_logic;
		WbErrAck 				: out  std_logic;
		WbBusy 					: out  std_logic;
		WbDataOut 				: out  std_logic_vector(31 downto 0);
      WbStatOut            : out  std_logic_vector(7 downto 0);
      WbStatIn             : in   std_logic_vector(31 downto 0);
-- Module clock and reset
      reset_o                : out  std_logic;   -- Module reset pulse from wishbone bus
      Sysclk_o               : out  std_logic;
-- Cfg Command strobes
      DetectI2C_o            : out  std_logic;  -- Pulsed to initiate detection of I2C buses
-- Temperature scanner Control signals
      tempScanEnable_o       : out  std_logic;   -- Enable temperature scanning and broadcasts
      tempScanRate_o         : out  std_logic_vector(9 downto 0);   -- Period between temperature sensor scans when scanning enabled - in units milliseconds
-- EEPROM register data control
      eepCmdReg_o            : out  std_logic_vector(16 downto 0);  -- command register - bit-31=readEEprom, bit-30=WriteEEprom, bit-29=Read Temps, bit-28=read SSN, bits(23:16)=Channel Address, bits(15:0)=Page address.
      eepCmdStatus_i         : in   std_logic_vector(3 downto 0);   -- Status for eeprom reads/writes - Bits(3:2)=read stat, bits(1:0)= write stat, "00"=not busy + no error, "01"=not busy + error, "10"=busy, "11"=busy + error.
      eepDataRdStb_i         : in   std_logic;  -- Pulsed from I2C manager to read write data writen by PAN to the register control into the I2C manager module
      eepDataWrtStb_i        : in   std_logic;  -- Pulsed from I2C manager to write data read from eeproms to the register control module
      eepDataRdWrtAddr_i     : in   std_logic_vector(4 downto 0); -- Address from the I2C manager to select which eepDataRegister to select for reading or writing.
      eepDataRegister_io     : inout   std_logic_vector(31 downto 0);  -- Sixteen data registers for reading / writing data from/to EEProms
-- Module data from I2C temperature and SSN bus reads
      LCB_temperature0_i     : in   std_logic_vector(11 downto 0);  -- Temperature of module from first sensor (address 0x48)
      LCB_temperature1_i     : in   std_logic_vector(11 downto 0);  -- Temperature of module from second sensor (address 0x49)
      PSM_temperature0_i     : in   std_logic_vector(11 downto 0);
      PSM_temperature1_i     : in   std_logic_vector(11 downto 0);
      TSM_temperature0_i     : in   std_logic_vector(11 downto 0);
      TSM_temperature1_i     : in   std_logic_vector(11 downto 0);
      AFE0_temperature0_i    : in   std_logic_vector(11 downto 0);
      AFE0_temperature1_i    : in   std_logic_vector(11 downto 0);
      AFE1_temperature0_i    : in   std_logic_vector(11 downto 0);
      AFE1_temperature1_i    : in   std_logic_vector(11 downto 0);
      LCB_SSN_i              : in   std_logic_vector(31 downto 0);  -- Silicon serial number of LCB module at address 0x50
      PSM_SSN_i              : in   std_logic_vector(31 downto 0);
      TSM_SSN_i              : in   std_logic_vector(31 downto 0);
      AFE0_SSN_i             : in   std_logic_vector(31 downto 0);
      AFE1_SSN_i             : in   std_logic_vector(31 downto 0);
-- Shutter and preflash control
      shutterEnable_o        : out  std_logic;  -- Enables shutter operation controlled by the sequencer
      shutterOpen_o          : out  std_logic;  -- Force shutter open command
      preflashEnable_o       : out  std_logic;  -- Enables preflash operation by the sequencer
      preflashOn_o           : out  std_logic;  -- Force preflash on command
      shutterStatusForce_o   : out  std_logic;  -- Force overide the of the shutter status input to show status according to commanded shutter state
      shutterPolarity_o      : out  std_logic;  -- sets the output polarity of the shutter drive signal
      preflashPolarity_o     : out  std_logic;  -- sets the output polarity of the preflash signal
      shutterStatPolarity_o  : out  std_logic;  -- sets the shutter status signal polarity at the shutter connector. 0=low true status signal (opto-isolator not turned on); 1=High true status signal (opto-isolator turned on).
      shutterOpenTime_i      : in   std_logic_vector(9 downto 0); -- Time in milliseconds from shutter open command to receiving shutter open status
      shutterCloseTime_i     : in   std_logic_vector(9 downto 0); -- Time in milliseconds from shutter close command to receiving shutter close status
      shutterStatus_i        : in   std_logic_vector(1 downto 0); -- Shutter status input from hardware - 00 = forced status, 01 = shutter open, 10 = shutter closed, 11 = shutter error.
-- Sequencer control
      sequencerEnable_o      : out  std_logic;   -- Sequencer enabled
      DheIsMaster_o          : out  std_logic;   -- Designates this DHE as a master when true.
      IntegrationTime_o      : out  std_logic_vector(31 downto 0);   -- Required integration time (ms).
      IntegrationCntr_i      : in   std_logic_vector(31 downto 0);   -- Actual integration counter - accumulative integration time in ms.
      PauseExposure_o        : out  std_logic;                       -- Pause the integration counter
      loopReg00_o            : out  std_logic_vector(15 downto 0);   -- 16 loop registers for sequencer branch control
      loopReg01_o            : out  std_logic_vector(15 downto 0);
      loopReg02_o            : out  std_logic_vector(15 downto 0);
      loopReg03_o            : out  std_logic_vector(15 downto 0);
      loopReg04_o            : out  std_logic_vector(15 downto 0);
      loopReg05_o            : out  std_logic_vector(15 downto 0);
      loopReg06_o            : out  std_logic_vector(15 downto 0);
      loopReg07_o            : out  std_logic_vector(15 downto 0);
      loopReg08_o            : out  std_logic_vector(15 downto 0);
      loopReg09_o            : out  std_logic_vector(15 downto 0);
      loopReg10_o            : out  std_logic_vector(15 downto 0);
      loopReg11_o            : out  std_logic_vector(15 downto 0);
      loopReg12_o            : out  std_logic_vector(15 downto 0);
      loopReg13_o            : out  std_logic_vector(15 downto 0);
      loopReg14_o            : out  std_logic_vector(15 downto 0);
      loopReg15_o            : out  std_logic_vector(15 downto 0);
      SyncDelay_o            : out  std_logic_vector(7 downto 0);    -- Delay (in ms) for sequencer to wait before sending sync pulse to slaves when in master mode
      UserBits_o             : out  std_logic_vector(3 downto 0);    -- Sequencer command register user bits
      SeqClkDiv_o            : out  std_logic_vector(1 downto 0);    -- Sequencer clock divider factor
      PatternMemData_i       : in   std_logic_vector(15 downto 0);   -- Sequencer pattern memory data for reads
      ProgramMemData_i       : in   std_logic_vector(15 downto 0);   -- Sequencer program memory data for reads
      WrtPatMem_o            : out  std_logic;                       -- Write strobe for sequencer pattern memory block
      WrtPgmMem_o            : out  std_logic;                       -- Write strobe for sequencer program memory block
      MemAddr_o              : out  std_logic_vector(11 downto 0);   -- Read / write Address to sequencer memory blocks
      MemData_o              : out  std_logic_vector(15 downto 0);   -- Write data to sequencer memeory blocks
      SeqCmdReg_i            : in   std_logic_vector(15 downto 0);   -- copy of the command register that controls the sequencer
      SeqStatus_i            : in   std_logic_vector(31 downto 0);   -- Sequencer status word
      SeqEnablesReg_i        : in   std_logic_vector(7 downto 0);    -- Sequencer enables register 
      SeqTestCondReg_i       : in  std_logic_vector(31 downto 0);    -- Sequencer test condition register - read only
      FrameStart_i           : in  std_logic;                        -- Start of new frame flag
      LineStart_i            : in  std_logic;                        -- Start of new row flag
-- Local status data
      PSM_ModuleDetect_i     : in   std_logic;
      TSM_ModuleDetect_i     : in   std_logic;
      AFE0_ModuleDetect_i    : in   std_logic;
      AFE1_ModuleDetect_i    : in   std_logic;
      ReadoutActive_i        : in   std_logic;
      TempChanSlct_o         : out  std_logic_vector(2 downto 0);
      BusEventStatus_i       : in  std_logic_vector(3 downto 0);
      BusErrorReg_i          : in  std_logic_vector(3 downto 0);
      BusGrantReg_i          : in  std_logic_vector(3 downto 0);
      BusTimeoutReg_i        : in  std_logic_vector(3 downto 0)
 	);

end CFG_RegisterControl_V220;

architecture Behavioral of CFG_RegisterControl_V220 is

-- Firmware module code revision value
   constant CODE_IDENT   : std_logic_vector(15 downto 0) := x"00DC"; -- Current firmware version is 2.20
   constant MODULE_IDENT : std_logic_vector(15 downto 0) := x"00CB"; -- CFG Module identity code is 203
   
-- eepromCmdReg command codes

   constant READ_EEPROM_CMD  : std_logic_vector(3 downto 0) := "1000";      -- Read eeprom pages  
   constant WRITE_EEPROM_CMD : std_logic_vector(3 downto 0) := "0100";      -- Write eeprom pages  
   constant READ_TEMPS_CMD   : std_logic_vector(3 downto 0) := "0010";      -- Read temperature sensors
   constant READ_SSN_CMD     : std_logic_vector(3 downto 0) := "0001";      -- Read silicon serial numbers

-- Misc bits --
   signal AddrErr : std_logic;      -- Indicates an addressing mismatch in this module i.e. bus master is trying to write to a non existant register

-- Decoding Vectors
   signal CS16K_0, CS16K_1, CS16K_2, CS16K_3 : std_logic;
   signal CS4K_0, CS4K_1, CS4K_2, CS4K_3 : std_logic;
   signal CS1K_0, CS1K_1, CS1K_2, CS1K_3 : std_logic;
   signal CS256_0, CS256_1, CS256_2, CS256_3 : std_logic;
   signal CS64_0, CS64_1, CS64_2, CS64_3 : std_logic;
   signal CS16_0, CS16_1, CS16_2, CS16_3 : std_logic;
   signal CS_0, CS_1, CS_2, CS_3, CS_4, CS_5, CS_6, CS_7 : std_logic;
   signal CS_8, CS_9, CS_A, CS_B, CS_C, CS_D, CS_E, CS_F : std_logic;

-- Misc registers and selects
   signal CodeId_cs, ModuleId_cs : std_logic;
   signal CfgServModInStatus_cs, CfgServModOutStatus_cs : std_logic;
	signal Address : std_logic_vector(15 downto 0);
	signal Data_in : std_logic_vector(31 downto 0);
   signal LocalStatus_reg : std_logic_vector(31 downto 0);
   signal cfgReg00_cs, cfgReg01_cs, cfgReg02_cs, cfgReg03_cs, cfgReg04_cs, cfgReg05_cs, cfgReg06_cs, cfgReg07_cs : std_logic;
   signal cfgReg08_cs, cfgReg09_cs, cfgReg10_cs, cfgReg11_cs, cfgReg12_cs, cfgReg13_cs, cfgReg14_cs, cfgReg15_cs : std_logic;
   signal mcbControl_cs  : std_logic;
   signal mcbControl_reg : std_logic_vector(15 downto 0);
   
-- Address select signals for the Application Interface module
   signal tempScanEnable_cs, tempScanRate_cs : std_logic;
   signal eepRdCmdReg_cs, eepWrtCmdReg_cs : std_logic;
   signal eepDataRegister00_cs, eepDataRegister01_cs, eepDataRegister02_cs, eepDataRegister03_cs : std_logic;
   signal eepDataRegister04_cs, eepDataRegister05_cs, eepDataRegister06_cs, eepDataRegister07_cs : std_logic;
   signal eepDataRegister08_cs, eepDataRegister09_cs, eepDataRegister10_cs, eepDataRegister11_cs : std_logic;
   signal eepDataRegister12_cs, eepDataRegister13_cs, eepDataRegister14_cs, eepDataRegister15_cs : std_logic;
   signal LCB_temperature0_cs, LCB_temperature1_cs, PSM_temperature0_cs, PSM_temperature1_cs, TSM_temperature0_cs : std_logic;
   signal TSM_temperature1_cs, AFE0_temperature0_cs, AFE0_temperature1_cs, AFE1_temperature0_cs, AFE1_temperature1_cs : std_logic;
   signal LCB_SSN_cs, PSM_SSN_cs, TSM_SSN_cs, AFE0_SSN_cs, AFE1_SSN_cs : std_logic;
   signal shutterEnable_cs, shutterOpen_cs, preflashEnable_cs, preflashOn_cs, shutterStatusForce_cs, shutterOpenTime_cs : std_logic;
   signal shutterCloseTime_cs, shutterStatus_cs : std_logic;
   signal shutterPol_cs, preflashPol_cs, shutterStatPol_cs : std_logic;
   signal sequencerEnable_cs, DheIsSlave_cs : std_logic;
   signal DetectI2C_cs, readTemps_cs, readSsn_cs : std_logic;

-- Sequencer register selects
   signal loopReg00_cs, loopReg01_cs, loopReg02_cs, loopReg03_cs, loopReg04_cs, loopReg05_cs, loopReg06_cs, loopReg07_cs : std_logic;
   signal loopReg08_cs, loopReg09_cs, loopReg10_cs, loopReg11_cs, loopReg12_cs, loopReg13_cs, loopReg14_cs, loopReg15_cs : std_logic;
   signal IntegrationTime_cs, IntegrationCntr_cs, PauseExposure_cs : std_logic;
   signal SyncDelay_cs, UserBits_cs, SeqClkDiv_cs, PatMem_cs, PgmMem_cs : std_logic;
   signal SeqCmdReg_cs, SeqStatus_cs, SeqEnables_cs, SeqTestReg_cs : std_logic;
   
-- Registers for the application interface module
   signal tempScanEnable_reg       : std_logic;
   signal tempScanRate_reg         : std_logic_vector(9 downto 0);
   signal eepCmdReg_reg            : std_logic_vector(16 downto 0);
   signal eepDataRegister00_reg, eepDataRegister01_reg, eepDataRegister02_reg, eepDataRegister03_reg : std_logic_vector(31 downto 0);
   signal eepDataRegister04_reg, eepDataRegister05_reg, eepDataRegister06_reg, eepDataRegister07_reg : std_logic_vector(31 downto 0);
   signal eepDataRegister08_reg, eepDataRegister09_reg, eepDataRegister10_reg, eepDataRegister11_reg : std_logic_vector(31 downto 0);
   signal eepDataRegister12_reg, eepDataRegister13_reg, eepDataRegister14_reg, eepDataRegister15_reg : std_logic_vector(31 downto 0);
   signal shutterEnable_reg        : std_logic;
   signal shutterOpen_reg          : std_logic;
   signal preflashEnable_reg       : std_logic;
   signal preflashOn_reg           : std_logic;
   signal shutterStatusForce_reg   : std_logic;
   signal sequencerEnable_reg      : std_logic;
   signal shutterPol_reg           : std_logic;
   signal preflashPol_reg          : std_logic;
   signal shutterStatPol_reg       : std_logic;
   
--   signal DheSlaveClkSwitchEn_reg  : std_logic;
   signal DetectI2C_reg            : std_logic_vector(4 downto 0);

-- Registers for the sequencer
   signal loopDataReg00_reg, loopDataReg01_reg, loopDataReg02_reg, loopDataReg03_reg : std_logic_vector(15 downto 0);
   signal loopDataReg04_reg, loopDataReg05_reg, loopDataReg06_reg, loopDataReg07_reg : std_logic_vector(15 downto 0);
   signal loopDataReg08_reg, loopDataReg09_reg, loopDataReg10_reg, loopDataReg11_reg : std_logic_vector(15 downto 0);
   signal loopDataReg12_reg, loopDataReg13_reg, loopDataReg14_reg, loopDataReg15_reg : std_logic_vector(15 downto 0);
   signal SyncDelay_reg            : std_logic_vector(7 downto 0);
   signal UserBits_reg             : std_logic_vector(3 downto 0);
   signal IntegrationTime_reg      : std_logic_vector(31 downto 0);
   signal SeqClkDiv_reg            : std_logic_vector(1 downto 0);
   signal PauseExposure_reg        : std_logic;
   signal DheIsSlave_reg           : std_logic;
   
begin
------------------------------------------------------------------------
-- Combinational and Static logic assignments
------------------------------------------------------------------------
	Address <= WbAddress;
	Data_in <= WbDataIn;
	WbBusy <= '0';									-- Not used
	WbWrtAck <= WbWriteEnable when (AddrErr = '0' and WbWriteEnable = '1') else '0';
	WbErrAck <= AddrErr when (WbWriteEnable = '1') else '0';
   LocalStatus_reg <= BusEventStatus_i & BusErrorReg_i & BusGrantReg_i & BusTimeoutReg_i & "0000000000" & LineStart_i & FrameStart_i & DheIsSlave_reg & AFE1_ModuleDetect_i & AFE0_ModuleDetect_i & ReadoutActive_i;
   reset_o  <= WbReset;
   Sysclk_o <= WbSysClk;
   
-- Status bit assignments 
-- Output
   WbStatOut(0) <= ReadoutActive_i;
   WbStatOut(1) <= AFE0_ModuleDetect_i;
   WbStatOut(2) <= AFE1_ModuleDetect_i;
   WbStatOut(3) <= DheIsSlave_reg;
   WbStatOut(4) <= FrameStart_i;
   WbStatOut(5) <= LineStart_i;
   WbStatOut(6) <= '0';
   WbStatOut(7) <= '0';
-- Input
   TempChanSlct_o(0) <= WbStatIn(0);
   TempChanSlct_o(1) <= WbStatIn(1);
   TempChanSlct_o(2) <= WbStatIn(2);

------------------------------------------------------------------------
-- Module output port assignments
------------------------------------------------------------------------
   tempScanEnable_o       <= tempScanEnable_reg and not ReadoutActive_i;  -- Suspend temperature scan activity while readout is in progress.      
   tempScanRate_o         <= tempScanRate_reg;
   eepCmdReg_o            <= eepCmdReg_reg;
   shutterEnable_o        <= shutterEnable_reg;       
   shutterOpen_o          <= shutterOpen_reg;         
   preflashEnable_o       <= preflashEnable_reg;      
   preflashOn_o           <= preflashOn_reg;          
   shutterStatusForce_o   <= shutterStatusForce_reg;  
   sequencerEnable_o      <= sequencerEnable_reg;     
   shutterPolarity_o      <= shutterPol_reg;
   preflashPolarity_o     <= preflashPol_reg;
   shutterStatPolarity_o  <= shutterStatPol_reg;
   DheIsMaster_o          <= not DheIsSlave_reg;
   eepDataRegister_io     <= (others => 'Z') when (eepDataRdStb_i = '0') else
                              eepDataRegister00_reg when (eepDataRdWrtAddr_i = "00000") else
                              eepDataRegister01_reg when (eepDataRdWrtAddr_i = "00001") else
                              eepDataRegister02_reg when (eepDataRdWrtAddr_i = "00010") else
                              eepDataRegister03_reg when (eepDataRdWrtAddr_i = "00011") else
                              eepDataRegister04_reg when (eepDataRdWrtAddr_i = "00100") else
                              eepDataRegister05_reg when (eepDataRdWrtAddr_i = "00101") else
                              eepDataRegister06_reg when (eepDataRdWrtAddr_i = "00110") else
                              eepDataRegister07_reg when (eepDataRdWrtAddr_i = "00111") else
                              eepDataRegister08_reg when (eepDataRdWrtAddr_i = "01000") else
                              eepDataRegister09_reg when (eepDataRdWrtAddr_i = "01001") else
                              eepDataRegister10_reg when (eepDataRdWrtAddr_i = "01010") else
                              eepDataRegister11_reg when (eepDataRdWrtAddr_i = "01011") else
                              eepDataRegister12_reg when (eepDataRdWrtAddr_i = "01100") else
                              eepDataRegister13_reg when (eepDataRdWrtAddr_i = "01101") else
                              eepDataRegister14_reg when (eepDataRdWrtAddr_i = "01110") else
                              eepDataRegister15_reg when (eepDataRdWrtAddr_i = "01111") else
                              (others => 'Z');
   IntegrationTime_o      <= IntegrationTime_reg;
   PauseExposure_o        <= PauseExposure_reg;
   loopReg00_o            <= loopDataReg00_reg;
   loopReg01_o            <= loopDataReg01_reg;
   loopReg02_o            <= loopDataReg02_reg;
   loopReg03_o            <= loopDataReg03_reg;
   loopReg04_o            <= loopDataReg04_reg;
   loopReg05_o            <= loopDataReg05_reg;
   loopReg06_o            <= loopDataReg06_reg;
   loopReg07_o            <= loopDataReg07_reg;
   loopReg08_o            <= loopDataReg08_reg;
   loopReg09_o            <= loopDataReg09_reg;
   loopReg10_o            <= loopDataReg10_reg;
   loopReg11_o            <= loopDataReg11_reg;
   loopReg12_o            <= loopDataReg12_reg;
   loopReg13_o            <= loopDataReg13_reg;
   loopReg14_o            <= loopDataReg14_reg;
   loopReg15_o            <= loopDataReg15_reg;
   SyncDelay_o            <= SyncDelay_reg;           
   UserBits_o             <= UserBits_reg; 
   SeqClkDiv_o            <= SeqClkDiv_reg;
   WrtPatMem_o            <= PatMem_cs and WbWriteEnable;
   WrtPgmMem_o            <= PgmMem_cs and WbWriteEnable;
   MemAddr_o              <= Address(11 downto 0); -- Address to sequencer memory blocks 
   MemData_o              <= Data_in(15 downto 0); 
   DetectI2C_o            <= DetectI2C_cs and WbWriteEnable;
   DetectI2C_reg          <= AFE1_ModuleDetect_i & AFE0_ModuleDetect_i & TSM_ModuleDetect_i & PSM_ModuleDetect_i & '1';
   
   eepCmdsProc : process(WbSysClk)
   begin
      if (rising_edge(WbSysClk)) then
         if (WbReset = '1') then
            eepCmdReg_reg <= (others => '0');
         elsif (WbWriteEnable = '1') then
            if (eepRdCmdReg_cs = '1') then
               eepCmdReg_reg <= READ_EEPROM_CMD & Data_in(20 downto 16) & Data_in(7 downto 0);   
            elsif (eepWrtCmdReg_cs = '1') then
               eepCmdReg_reg <= WRITE_EEPROM_CMD & Data_in(20 downto 16) & Data_in(7 downto 0);   
            elsif (readTemps_cs = '1') then
               eepCmdReg_reg <= READ_TEMPS_CMD & Data_in(20 downto 16) & "0000000" & Data_In(0);   
            elsif (readSsn_cs = '1') then
               eepCmdReg_reg <= READ_SSN_CMD & Data_in(20 downto 16) & "00000000";   
            end if;
         else
            eepCmdReg_reg <= eepCmdReg_reg and "00001111111111111";  -- Cancel the command strobes
         end if;
      end if;
   end process;

------------------------------------------------------------------------
-- MODULE ADDRESS DECODE LOGIC. Enable the decoded address strobes as required.
------------------------------------------------------------------------
-- Decode 16K word Blocks
    CS16K_0 <= (not Address(15)) and (not Address(14));   -- 0x0000 => 0x3FFF
    CS16K_1 <= (not Address(15)) and Address(14);         -- 0x4000 => 0x7FFF
    CS16K_2 <= Address(15) and (not Address(14));         -- 0x8000 => 0xBFFF
    CS16K_3 <= Address(15) and Address(14);               -- 0xC000 => 0xFFFF

-- Decode 4K word Blocks
    CS4K_0 <= (not Address(13)) and (not Address(12));    --0x0000 => 0x0FFF, 0x4000 => 0x4FFF, 0x8000 => 0x8FFF, 0xC000 => 0xCFFF
    CS4K_1 <= (not Address(13)) and Address(12);          --0x1000 => 0x1FFF, 0x5000 => 0x5FFF, 0x9000 => 0x9FFF, 0xD000 => 0xDFFF
    CS4K_2 <= Address(13) and (not Address(12));          --0x2000 => 0x2FFF, 0x6000 => 0x6FFF, 0xA000 => 0xAFFF, 0xE000 => 0xEFFF
    CS4K_3 <= Address(13) and Address(12);                -- 0x3000 => 0x3FFF, 0x7000 => 0x7FFF, 0xB000 => 0xBFFF, 0xF000 => 0xFFFF

-- Decode 1K word Blocks
    CS1K_0 <= (not Address(11)) and (not Address(10));   -- 0x_000 => 0x_3FF
    CS1K_1 <= (not Address(11)) and Address(10);         -- 0x_400 => 0x_7FF
    CS1K_2 <= Address(11) and (not Address(10));         -- 0x_800 => 0x_BFF
    CS1K_3 <= Address(11) and Address(10);               -- 0x_C00 => 0x_FFF

-- Decode 256 word Blocks
    CS256_0 <= (not Address(9)) and (not Address(8));    --0x_000 => 0x_0FF, 0x_400 => 0x_4FF, 0x_800 => 0x_8FF, 0x_C00 => 0x_CFF
    CS256_1 <= (not Address(9)) and Address(8);          --0x_100 => 0x_1FF, 0x_500 => 0x_5FF, 0x_900 => 0x_9FF, 0x_D00 => 0x_DFF
    CS256_2 <= Address(9) and (not Address(8));          --0x_200 => 0x_2FF, 0x_600 => 0x_6FF, 0x_A00 => 0x_AFF, 0x_E00 => 0x_EFF
    CS256_3 <= Address(9) and Address(8);                -- 0x_300 => 0x_3FF, 0x_700 => 0x_7FF, 0x_B00 => 0x_BFF, 0x_F00 => 0x_FFF

-- Decode 64 word Blocks
    CS64_0 <= (not Address(7)) and (not Address(6));   -- 0x__00 => 0x__3F
    CS64_1 <= (not Address(7)) and Address(6);         -- 0x__40 => 0x__7F
    CS64_2 <= Address(7) and (not Address(6));         -- 0x__80 => 0x__BF
    CS64_3 <= Address(7) and Address(6);               -- 0x__C0 => 0x__FF

 -- Decode 16 word Blocks
    CS16_0 <= (not Address(5)) and (not Address(4));    --0x__00 => 0x__0F, 0x__40 => 0x__4F, 0x__80 => 0x__8F, 0x__C0 => 0x__CF
    CS16_1 <= (not Address(5)) and Address(4);          --0x__10 => 0x__1F, 0x__50 => 0x__5F, 0x__90 => 0x__9F, 0x__D0 => 0x__DF
    CS16_2 <= Address(5) and (not Address(4));          --0x__20 => 0x__2F, 0x__60 => 0x__6F, 0x__A0 => 0x__AF, 0x__E0 => 0x__EF
    CS16_3 <= Address(5) and Address(4);                -- 0x__30 => 0x__3F, 0x__70 => 0x__7F, 0x__B0 => 0x__BF, 0x__F0 => 0x__FF

-- Decode single word address lines
    CS_0 <= (not Address(3)) and (not Address(2)) and (not Address(1)) and (not Address(0));
    CS_1 <= (not Address(3)) and (not Address(2)) and (not Address(1)) and Address(0);
    CS_2 <= (not Address(3)) and (not Address(2)) and Address(1) and (not Address(0));
    CS_3 <= (not Address(3)) and (not Address(2)) and Address(1) and Address(0);
    CS_4 <= (not Address(3)) and Address(2) and (not Address(1)) and (not Address(0));
    CS_5 <= (not Address(3)) and Address(2) and (not Address(1)) and Address(0);
    CS_6 <= (not Address(3)) and Address(2) and Address(1) and (not Address(0));
    CS_7 <= (not Address(3)) and Address(2) and Address(1) and Address(0);
    CS_8 <= Address(3) and (not Address(2)) and (not Address(1)) and (not Address(0));
    CS_9 <= Address(3) and (not Address(2)) and (not Address(1)) and Address(0);
    CS_A <= Address(3) and (not Address(2)) and Address(1) and (not Address(0));
    CS_B <= Address(3) and (not Address(2)) and Address(1) and Address(0);
    CS_C <= Address(3) and Address(2) and (not Address(1)) and (not Address(0));
    CS_D <= Address(3) and Address(2) and (not Address(1)) and Address(0);
    CS_E <= Address(3) and Address(2) and Address(1) and (not Address(0));
    CS_F <= Address(3) and Address(2) and Address(1) and Address(0);
    
    cfgReg00_cs <= '1' when (eepDataRdWrtAddr_i = "00000") else '0';
    cfgReg01_cs <= '1' when (eepDataRdWrtAddr_i = "00001") else '0';
    cfgReg02_cs <= '1' when (eepDataRdWrtAddr_i = "00010") else '0';
    cfgReg03_cs <= '1' when (eepDataRdWrtAddr_i = "00011") else '0';
    cfgReg04_cs <= '1' when (eepDataRdWrtAddr_i = "00100") else '0';
    cfgReg05_cs <= '1' when (eepDataRdWrtAddr_i = "00101") else '0';
    cfgReg06_cs <= '1' when (eepDataRdWrtAddr_i = "00110") else '0';
    cfgReg07_cs <= '1' when (eepDataRdWrtAddr_i = "00111") else '0';
    cfgReg08_cs <= '1' when (eepDataRdWrtAddr_i = "01000") else '0';
    cfgReg09_cs <= '1' when (eepDataRdWrtAddr_i = "01001") else '0';
    cfgReg10_cs <= '1' when (eepDataRdWrtAddr_i = "01010") else '0';
    cfgReg11_cs <= '1' when (eepDataRdWrtAddr_i = "01011") else '0';
    cfgReg12_cs <= '1' when (eepDataRdWrtAddr_i = "01100") else '0';
    cfgReg13_cs <= '1' when (eepDataRdWrtAddr_i = "01101") else '0';
    cfgReg14_cs <= '1' when (eepDataRdWrtAddr_i = "01110") else '0';
    cfgReg15_cs <= '1' when (eepDataRdWrtAddr_i = "01111") else '0';

----------------------------------------------------------------------------------------------------
-- REGISTER ADDRESS DECODE LOGIC. Use the decoded address strobes to define a particular address.
----------------------------------------------------------------------------------------------------
-- From TRNT-AD-08-0003-Rev1.1
-- Each register description line will be a comment in the FPGA source code file of the form below. 
-- A comment  line  may be included in the file as a column header line.  
-- The column header comment line will look like this:
--
--##AD: EPAdr: AtrName: ArraySz: #Vals: TypCde: BdAdr: RegAdr: AdrIncr: Size: SetMthd: RdMthd: SigTyp: DspGrp: dfltVal: Units: Cmnt
--
---#AD is the tag indication for a normal register description line
-- EPAdr  This field will contain the starting word address (0-4095) in the EEPROM of the configuration information for this attribute. Acceptable values are an address starting at 0xFFF down to 0x000 in hex or decimal format or the words AUTO ,UNDCD or NONE. The AUTO / UNDCD words will indicate to assimilate that the location of the configuration data block for this attribute should be assigned automatically. The NONE word indicates to assimilate that no EEPROM words should be assigned to this attribute.
-- AtrName  The name used/defined in the FPGA. This is the DHE hardware name of the register/function.
-- ArraySz  the number of Registers in an array of registers with a common name and purpose. This can always be 1 treating each register in an array as a single register. The PAN already always unwraps arrays of registers to single registers.
-- #Vals  gives the size in words of the values needed to describe the behavior of the register. The values occupy EEPAdr  EEPAdr+(#Values-1)) in the EEPROM. These words contain the values described by TypCde below.
-- TypCde (Type Codes)
--       A  A single integer value entry representing information about the EEPROM structure or provenance; or
--             a single word engineering value stored as a 32 BIT binary value to be loaded automatically by the FPGA startup code; 
--       B  Two(if read only) or four(if read/write) integer values that describe a bit field register. Laid out as >Width, Mask, Max, Min< The Bit field is always assumed to be located in bits 0  width
--       C  Two(if read only) or four(if read/write) Floating point values that describe a value register. Laid out as >Slope,Intercept,Max,Min<
--       D  a single floating point value used in a software constant
--       E  a Four/Two word entry representing a checksum value least significant word first.
-- MdlName  the text name for the board/module containing the Register eg, CCD1, LCB
-- RegAddr: the 16 bit address of the register in the module address space of the first register in the group. The PAN will treat BdAddr and RegAddress: fields as a single address by summing the two fields with DheRegAddress = ((BdAddr<<16)&RegAddr+AdrIncr() ) The csv file will place a !BDNAME:0x04 line at the beginning of the file and describe the address as ,BDNAME:0234, in the register address field.
-- AdrIncr  A code indicating how addresses increment in arrays of registers. Three values are recognized at this time.
--       0, 1  indicate the addresses increment by 1 for each successive index i.e.. 1 2 3 4 5, etc.
--       2 - indicates the addresses increment by shifting the lower #inArray bits by one bit to the left. i.e.  0x1101, 0x1102, 0x1104, 0x1108, 0x1110, etc.
-- Size  The width in bits of the register, usually 32. Reasonable values are 1, 8, 12, 16, 24, 32.
-- SetMthd, RdMthd  The method used to write/read the register. Three methods are known:
--       NONE  The attribute cannot be accessed for writing or for reading.
--       SMPL  The register is accessed with a simple write or read command.
--       RDMW  The register is a bit field that requires a read, masking the bits that are unchanged and a simple write to set the register. On read the value returned is the value of the bits requested.
-- SigTyp  This gives the type of signal for connection purposes. The fpConfig routine will use this to limit the connection type of detector signals for this attribute. fpConfig will recognize the following signal types. Note: These types refer to the connections to the detector or Dewar and are not used for internal or logical function attributes. These attributes should be given the NONE, LGCL or SFTWR signal type.
--       DPWR  a detector power signal  a voltage usually connected to a fixed bias level.
--       DCLK  detector clock signal  a control clock for the detector. These break down into an UpperRail, a LowerRail and a logical level. Connecting any one of the three signals to a detector pin automatically connects the other two signals.
--       AGND  a detector analog ground signal  
--       OGND  other detector ground signals  
--       VOUT  a detector video output. This breaks out into the video signal and an offset voltage for that signal.
--       BIAS - a detector bias voltage.
--       DVLT  a detector voltage not otherwise classified (e.g. logical controls as in OTAs. or hardware gain controls).
--       NONE  an attribute that does not connect to the detector and for which no or only a write/read test exists. 
--       CMND  an attribute that when written to results in the start of some process in the DHE,  eg. afe1Reset, sysReset
--       TELE  an attribute that reads back the value of some telemetry voltage.
--       CNFG  a configuration Attribute  eg. Afe1TpGrp2Cfg or CcdAmpCfg
--       CNST  attributes that contain constants eg. CodeID, AFE1SerNum etcz
--       LGCL - an attribute that is a logical connection to the detector (i.e. rowCount, Number of outputs, pipeline enables, etc.)
--       TEMP  a temperature control signal.
--       SFTWR  connects to Dewar only through other values 
-- DspGrp  a number representing the display group that includes this attribute. Equivalent to the MEC or BORG page number to be used for this attributes display.
-- DfltVal  the value assigned to the attribute register when the fpga is loaded on power-up or when a hard reset occurs.
-- Units - Engineering units used to describe the attribute
-- Cmnt  a comment describing the usage/purpose/function of this register.
-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- ATTRIBUTE DESCRIPTION:
-- #MD: CFG: 0x04:    --This is the Wishbone module select address of the Config service module registers

-- #AD: NONE: CfgCodeId: 1: 1: A:  CFG: 0xFFFF: 0: 16:  NONE: SMPL: CNST: 20: 2.02: Version: Firmware module revision code as MM.mm^MOD^ ^100.0^0.0^0.0^65535.0
   CodeId_cs <= CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_F;

-- #AD: NONE:  CfgModuleId: 1: 1: A: CFG: 0xFFFE: 0: 16: NONE:  SMPL: CNST: 20: 203.0: Ident: Firmware module identification code as an integer^MOD^ ^1.0^0.0^0.0^4294967295.0
-- #AD: NONE: CfgResetCmd: 1: 1: A:  CFG: 0xFFFE: 0: 0:  SMPL: NONE: CMND: 20: :ResetCmnd:Triggers an CFG module reset^MOD^ ^1.0^0.0^0.0^0.0
   ModuleId_cs <= CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_E;

-- #AD: NONE: CfgModInStatus: 1: 1: A: CFG: 0xFFFD: 0: 32:  NONE: SMPL: NONE: 20: 0: Boolean: Status of System as received from system.^MOD^ ^1.0^0.0^0.0^4294967295.0
   CfgServModInStatus_cs <= CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_D;

-- #AD: NONE: CfgModOutStatus: 1:  1: A: CFG: 0xFFFC: 0:  32:  NONE:  SMPL: NONE: 20: 0: Boolean: Status of CFG Control module as transmitted to system.^MOD^ ^1.0^0.0^0.0^4294967295.0
   CfgServModOutStatus_cs <= CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_C;

-- #AD: NONE: mcbControl: 1:  1: A: CFG: 0xFFFB: 0:  16:  NONE:  RDMSKWRT: RDMSKWRT: 20: 0: Boolean: Emulation of MNSN Orange master control board command reg.^MOD^ ^1.0^0.0^0.0^65535.0
   mcbControl_cs <= CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_B;

-- #AD: NONE: IntegrationTime: 1: 1: A: CFG: 0x0000: 0: 32: SMPL: SMPL:  CFNG: 1: 0: mSec: Sets the integration time of an exposure^MOD^ ^1.0^0.0^0.0^4294967295.0
   IntegrationTime_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_0 and CS_0;

-- #AD: NONE: TempScanEnable: 1: 1: A: CFG: 0x0010: 0: 1: SMPL: SMPL: CNFG: 20: 1: Boolean: Enable the board temperature measurement scanning function.^MOD^ ^1.0^0.0^0.0^1.0
   tempScanEnable_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_1 and CS_0;
   
-- #AD: NONE: TempScanPeriod: 1: 1: A: CFG: 0x0011: 0: 10: SMPL: SMPL: CNFG: 20: 300: mSec: Time period between temperature scans^MOD^ ^1.0^0.0^0.0^1023.0
   tempScanRate_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_1 and CS_1;

-- #AD: NONE: DetectI2CBus: 1: 1: A: CFG: 0x0012: 0: 5: SMPL: SMPL: CMND: 20: 0:Boolean:write - I2C detection; Read shows devices Bits (4;0)-Bits(AFE1, AFE0, TSM, PSM, LCB).^MOD^ ^1.0^0.0^0.0^31.0
   DetectI2C_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_1 and CS_2;

-- #AD: NONE: ReadTemps: 1: 1: A: CFG: 0x0013: 0: 1: SMPL: NONE: CMND: 20: 0: BitField: Write to initiate I2C temperature sensor reads^MOD^ ^1.0^0.0^65536.0^1048577.0
   readTemps_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_1 and CS_3;

-- #AD: NONE: ReadSerialNums: 1: 1: A: CFG: 0x0014: 0: 1: SMPL: NONE: CMND: 20: 0: BitField: Write to initiate I2C silicon serial number reads^MOD^ ^65536.0^0.0^1.0^16
   readSsn_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_1 and CS_4;

-- #AD: NONE: eepRdCmdReg: 1: 1: A: CFG: 0x0020: 0: 32: SMPL: SMPL: CMND: 20: 0: Boolean: Command register to select and read  an eeprom page^MOD^ ^1.0^0.0^0.0^4294967295.0
   eepRdCmdReg_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_2 and CS_0;
   
-- #AD: NONE: eepWrtCmdReg: 1: 1: A: CFG: 0x0021: 0: 32: SMPL: SMPL:  CMND: 20: 0: Boolean: Command register to select and write an eeprom page^MOD^ ^1.0^0.0^0.0^4294967295.0
   eepWrtCmdReg_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_2 and CS_1;
   
-- #AD: NONE: eepDataReg: 16: 1: A: CFG: 0x0030: 1: 32: SMPL: SMPL:  NONE: 20: 0: Binary: EEProm Data registers for read and write operations^MOD^ ^1.0^0.0^0.0^4294967295.0
-- #AD: NONE: eepFloatReg: 16: 4: C: CFG: 0x0030: 1: 32: SMPL: SMPL:  NONE: 20: 0: Value: EEProm Data registers for read and write operations^MOD^ ^1000.0^0.0^-2147482.0^2147482.0
   eepDataRegister00_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_0) or cfgReg00_cs;
   eepDataRegister01_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_1) or cfgReg01_cs;
   eepDataRegister02_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_2) or cfgReg02_cs;
   eepDataRegister03_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_3) or cfgReg03_cs;
   eepDataRegister04_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_4) or cfgReg04_cs;
   eepDataRegister05_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_5) or cfgReg05_cs;
   eepDataRegister06_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_6) or cfgReg06_cs;
   eepDataRegister07_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_7) or cfgReg07_cs;
   eepDataRegister08_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_8) or cfgReg08_cs;
   eepDataRegister09_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_9) or cfgReg09_cs;
   eepDataRegister10_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_A) or cfgReg10_cs;
   eepDataRegister11_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_B) or cfgReg11_cs;
   eepDataRegister12_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_C) or cfgReg12_cs;
   eepDataRegister13_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_D) or cfgReg13_cs;
   eepDataRegister14_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_E) or cfgReg14_cs;
   eepDataRegister15_cs <= (CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_3 and CS_F) or cfgReg15_cs;
   
-- #AD: UNDCD: LcbTemperature1: 1: 4: C: CFG: 0x0040: 0: 12: NONE: SMPL:  TEMP: 5: 0: Deg C: LCB Temperature reading from sensor 1^MOD^ ^16.0^0.0^-127.0^127.0  
   LCB_temperature0_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_0 and CS_0;
   
-- #AD: UNDCD: LcbTemperature2: 1: 4: C: CFG: 0x0041: 0: 12: NONE: SMPL:  TEMP: 5: 0: Deg C: LCB Temperature reading from sensor 2^MOD^ ^16.0^0.0^-127.0^127.0   
   LCB_temperature1_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_0 and CS_1;
   
-- #AD: UNDCD: PsmTemperature1: 1: 4: C: CFG: 0x0042: 0: 12: NONE: SMPL:  TEMP: 16: 0: Deg C: PSM Temperature reading from sensor 1^MOD^ ^16.0^0.0^-127.0^127.0    
   PSM_temperature0_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_0 and CS_2;
   
-- #AD: UNDCD: PsmTemperature2: 1: 4: C: CFG: 0x0043: 0: 12: NONE: SMPL:  TEMP: 16: 0: Deg C: PSM Temperature reading from sensor 2^MOD^ ^16.0^0.0^-127.0^127.0   
   PSM_temperature1_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_0 and CS_3;
   
-- #AD: UNDCD: TsmTemperature1: 1: 4: C: CFG: 0x0044: 0: 12: NONE: SMPL:  TEMP: 4: 0: Deg C: TSM Temperature reading from sensor 1^MOD^ ^16.0^0.0^-127.0^127.0    
   TSM_temperature0_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_0 and CS_4;
   
-- #AD: UNDCD: TsmTemperature2: 1: 4: C: CFG: 0x0045: 0: 12: NONE: SMPL:  TEMP: 4: 0: Deg C: TSM Temperature reading from sensor 2^MOD^ ^16.0^0.0^-127.0^127.0    
   TSM_temperature1_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_0 and CS_5;
   
-- #AD: UNDCD: Afe1Temperature1: 1: 4: C: CFG: 0x0046: 0: 12: NONE: SMPL:  TEMP: 8: 0: Deg C: AFE1 Temperature reading from sensor 1^MOD^ ^16.0^0.0^-127.0^127.0    
   AFE0_temperature0_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_0 and CS_6;
   
-- #AD: UNDCD: Afe1Temperature2: 1: 4: C: CFG: 0x0047: 0: 12: NONE: SMPL:  TEMP: 8: 0: Deg C: AFE1 Temperature reading from sensor 2^MOD^ ^16.0^0.0^-127.0^127.0    
   AFE0_temperature1_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_0 and CS_7;
   
-- #AD: UNDCD: Afe2Temperature1: 1: 4: C: CFG: 0x0048: 0: 12: NONE: SMPL:  TEMP: 8: 0: Deg C: AFE2 Temperature reading from sensor 1^MOD^ ^16.0^0.0^-127.0^127.0    
   AFE1_temperature0_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_0 and CS_8;
   
-- #AD: UNDCD: Afe2Temperature2: 1: 4: C: CFG: 0x0049: 0: 12: NONE: SMPL:  TEMP: 8: 0: Deg C: AFE2 Temperature reading from sensor 2^MOD^ ^16.0^0.0^-127.0^127.0    
   AFE1_temperature1_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_0 and CS_9;

-- #AD: NONE: LcbSerialNum: 1: 1: A: CFG: 0x0050: 0: 32: NONE: SMPL:  NONE: 5: 0: SerNum: LCB Silicon Serial Number^MOD^ ^1.0^0.0^0.0^4294967295.0 
   LCB_SSN_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_1 and CS_0;

-- #AD: NONE: PsmSerialNum: 1: 1: A: CFG: 0x0051: 0: 32: NONE: SMPL:  NONE: 16: 0: SerNum: PSM Silicon Serial Number^MOD^ ^1.0^0.0^0.0^4294967295.0  
   PSM_SSN_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_1 and CS_1;

-- #AD: NONE: TsmSerialNum: 1: 1: A: CFG: 0x0052: 0: 32: NONE: SMPL:  NONE: 4: 0: SerNum: TSM Silicon Serial Number^MOD^ ^1.0^0.0^0.0^4294967295.0  
   TSM_SSN_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_1 and CS_2;

-- #AD: NONE: Afe1SerialNum: 1: 1: A: CFG: 0x0053: 0: 32: NONE: SMPL:  NONE: 8: 0: SerNum: AFE1 Silicon Serial Number^MOD^ ^1.0^0.0^0.0^4294967295.0  
   AFE0_SSN_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_1 and CS_3;

-- #AD: NONE: Afe2SerialNum: 1: 1: A: CFG: 0x0054: 0: 32: NONE: SMPL:  NONE: 8: 0: SerNum: AFE2 Silicon Serial Number ^MOD^ ^1.0^0.0^0.0^4294967295.0 
   AFE1_SSN_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_1 and CS_4;

-- #AD: NONE: ShutterEnable: 1: 1: A: CFG: 0x0060: 0: 1: SMPL: SMPL: CNFG: 1: 1: Boolean: Enable the sequencer to control the shutter^MOD^ ^1.0^0.0^0.0^1.0
   shutterEnable_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_2 and CS_0;

-- #AD: NONE: ShutterOpenCmd: 1: 1: A: CFG: 0x0061: 0: 1: SMPL: SMPL: CMND: 1: 0: Boolean: Override the sequencer and force the shutter to open^MOD^ ^1.0^0.0^0.0^1.0
   shutterOpen_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_2 and CS_1;

-- #AD: NONE: PreflashEnable: 1: 1: A: CFG: 0x0062: 0: 1: SMPL: SMPL: CNFG: 1: 1: Boolean: Enable the sequencer to control the PreFlash^MOD^ ^1.0^0.0^0.0^1.0
   preflashEnable_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_2 and CS_2;
  
-- #AD: NONE: PreflashOnCmd: 1: 1: A: CFG: 0x0063: 0: 1: SMPL: SMPL: CMND: 1: 0: Boolean: Override the sequencer and force the preflash signal on^MOD^ ^1.0^0.0^0.0^1.0
   preflashOn_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_2 and CS_3;
   
-- #AD: NONE:  ShutterForceStatus: 1: 1: A: CFG: 0x0064: 0: 1: SMPL: SMPL: CNFG: 1: 1: Boolean: Override the status inputs and force status equal to the shutter cmd state^MOD^ ^1.0^0.0^0.0^1.0
   shutterStatusForce_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_2 and CS_4;

-- #AD: NONE: ShutterOpenTime: 1: 1: A: CFG: 0x0065: 0: 10: NONE: SMPL:  NONE: 1: 0: mSec: Time that shutter took to open after open shutter cmd^MOD^ ^1.0^0.0^0.0^1023.0
   shutterOpenTime_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_2 and CS_5;

-- #AD: NONE: ShutterCloseTime: 1: 1: A: CFG: 0x0066: 0: 10: NONE: SMPL:  NONE: 1: 0: mSec: Time that shutter took to close after close shutter cmd^MOD^ ^1.0^0.0^0.0^1023.0
   shutterCloseTime_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_2 and CS_6;

-- #AD: NONE: ShutterStatus: 1: 1: A: CFG: 0x0067: 0: 2: NONE: SMPL:  NONE: 1: 0: Boolean: Shutter hdwr status input - 00=forced status; 01=shutter open; 10=shutter closed, 11=shutter error.^MOD^ ^1.0^0.0^0.0^3.0
   shutterStatus_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_2 and CS_7;

-- #AD: NONE: ShutterPolarity: 1: 1: A: CFG: 0x0068: 0: 1: SMPL: SMPL:  CNFG: 1: 0: Boolean: Shutter hardware drive polarity - 0=low signal to open shutter; 1=High signal to open shutter^MOD^ ^1.0^0.0^0.0^1.0
   shutterPol_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_2 and CS_8;
   
-- #AD: NONE: PreflashPolarity: 1: 1: A: CFG: 0x0069: 0: 1: SMPL: SMPL:  CNFG: 1: 0: Boolean: Preflash hardware drive polarity - 0=low signal to turn on preflash; 1=High signal to turn on preflash^MOD^ ^1.0^0.0^0.0^1.0
   preflashPol_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_2 and CS_9;

-- #AD: NONE: ShutterStatusPolarity: 1: 1: A: CFG: 0x006A: 0: 1: SMPL: SMPL:  CNFG: 1: 0: Boolean: Shutter status hardware drive polarity - 0=low true status signal (opto-isolator not turned on); 1=High true status signal (opto-isolator turned on)at shutter connector^MOD^ ^1.0^0.0^0.0^1.0
   shutterStatPol_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_1 and CS16_2 and CS_A;
   
-- #AD: NONE: DheIsSlave: 1: 1: A: CFG: 0x80: 0: 1: SMPL: SMPL:  CNFG: 4: 0: Boolean: Configure the DHE as a slave device when true^MOD^ ^1.0^0.0^0.0^1.0
   DheIsSlave_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_2 and CS16_0 and CS_0;
   
-- Sequencer selects
-- #AD: NONE: SeqEnable: 1: 1: A: CFG: 0x100: 0: 1: SMPL: SMPL: CNFG: 6: 0: Boolean: Enable the primary sequencer to run code^MOD^ ^1.0^0.0^0.0^1.0
   sequencerEnable_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_0;
   
-- #AD: NONE: actualIntegrationTime: 1: 1: A: CFG: 0x0101: 0: 32: NONE: SMPL:  NONE: 1: 0: Sec: Reads the actual integration time of an exposure^MOD^ ^1000.0^0.0^0.0^4294967.0
   IntegrationCntr_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_1;
   
-- #AD: NONE: SeqCmds: 1: 1: A: CFG: 0x0102: 0: 1: NONE: SMPL:  NONE: 6: 0: Boolean: Current state of sequencer command register^MOD^ ^1.0^0.0^0.0^65535.0
   SeqCmdReg_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_2;
   
-- #AD: NONE: PauseExposure: 1: 1: A: CFG: 0x0103: 0: 1: SMPL: SMPL:  CMND: 1: 0: Boolean: Pauses the current exposure when true^MOD^ ^1.0^0.0^0.0^1.0
   PauseExposure_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_3;
   
-- #AD: NONE: SyncDelay: 1: 1: A: CFG: 0x0104: 0: 8: SMPL: SMPL:  CNFG: 6: 0: mSec: Sets the delay before a sync pulse is issued when acting as a master^MOD^ ^1.0^0.0^0.0^255.0
   SyncDelay_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_4;

-- #AD: NONE: SeqUserBits: 1: 1: A: CFG: 0x0105: 0: 4: SMPL: SMPL:  CNFG: 6: 0: Boolean: Sets the user bits (15;12) in the sequencer command register^MOD^ ^1.0^0.0^0.0^15.0
   UserBits_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_5;

-- #AD: NONE: SeqClkDivide: 1: 1: A: CFG: 0x0106: 0: 2: SMPL: SMPL:  CNFG: 6: 0: Boolean: Sets the sequencer clock divider ratio 2; 4; 8 or 16^MOD^ ^1.0^0.0^0.0^3.0
   SeqClkDiv_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_6;
      
-- #AD: NONE: SeqStatus: 1: 1: A: CFG: 0x0107: 0: 16: NONE: SMPL:  NONE: 6: 0: Boolean: Reads the sequencer status word^MOD^ ^1.0^0.0^0.0^4294967296.0
   SeqStatus_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_7;

-- #AD: NONE: SeqEFR: 1: 1: A: CFG: 0x0108: 0: 8: NONE: SMPL:  NONE: 6: 0: Boolean: Reads the sequencer enables register^MOD^ ^1.0^0.0^0.0^255.0
   SeqEnables_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_8;
   
-- #AD: NONE: SeqTST: 1: 1: A: CFG: 0x0109: 0: 32: NONE: SMPL:  NONE: 6: 0: Boolean: Reads the sequencer test condition register^MOD^ ^1.0^0.0^0.0^4294967296.0
   SeqTestReg_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_9;

-- #AD: NONE: SeqLoopReg: 16: 1: A: CFG: 0x0110: 1: 16: SMPL: SMPL:  CNFG: 6: 0: Count: Sequencer Loop Data Register for read and write operations^MOD^ ^1.0^0.0^0.0^65535.0
   loopReg00_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_0;
   loopReg01_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_1;
   loopReg02_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_2;
   loopReg03_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_3;
   loopReg04_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_4;
   loopReg05_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_5;
   loopReg06_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_6;
   loopReg07_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_7;
   loopReg08_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_8;
   loopReg09_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_9;
   loopReg10_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_A;
   loopReg11_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_B;
   loopReg12_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_C;
   loopReg13_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_D;
   loopReg14_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_E;
   loopReg15_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_1 and CS_F;
   
-- #AD: NONE: SeqPatMem: 1: 1: A: CFG: 0x1000: 1: 16: SMPL: SMPL:  CNFG: 6: 0: Code: Sequencer pattern memory space start address (Note actual mem size is 4096 locations)
   PatMem_cs <= CS16K_0 and CS4K_1;
   
-- #AD: NONE: SeqPgmMem: 1: 1: A: CFG: 0x4000: 1: 16: SMPL: SMPL:  CNFG: 6: 0: Code: Sequencer program memory space  start address (Note actual mem size is 1024 locations)
   PgmMem_cs <= CS16K_1 and CS4K_0 and CS1K_0; 
   
----------------------------------------------------------------------------------------------------
-- REGISTER DATA LATCHING LOGIC. Use the decoded address CS strobes to enable a register write.
----------------------------------------------------------------------------------------------------
   LcbCtrlRegLatchProc : process (WbSysClk)
      begin
         if (rising_edge(WbSysClk)) then
            if (WbReset = '1') then          -- Presets (default values)
               tempScanEnable_reg <= '1';    -- Temperature scanner on
               tempScanRate_reg <= "0100101100";  -- Nominal 300ms between scans
               shutterEnable_reg <= '1';          -- Shutter is enabled
               shutterOpen_reg <= '0';            -- But closed after reset
               preflashEnable_reg <= '1';         -- Preflash is also enabled
               preflashOn_reg <= '0';             -- But off
               shutterStatusForce_reg <= '1';     -- Force status since we may not have a remote status sensor
               sequencerEnable_reg <= '0';        -- is off .. this is a reset .. something mighty is going down.
               DheIsSlave_reg <= '0';             -- Who isn't - But we're not until otherwise told.
               IntegrationTime_reg <= (others => '0');
               PauseExposure_reg <= '0';          -- If we ever were paused
             elsif (eepDataWrtStb_i = '1') then
               if (eepDataRegister00_cs = '1') then
                  eepDataRegister00_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister01_cs = '1') then
                  eepDataRegister01_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister02_cs = '1') then
                  eepDataRegister02_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister03_cs = '1') then
                  eepDataRegister03_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister04_cs = '1') then
                  eepDataRegister04_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister05_cs = '1') then
                  eepDataRegister05_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister06_cs = '1') then
                  eepDataRegister06_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister07_cs = '1') then
                  eepDataRegister07_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister08_cs = '1') then
                  eepDataRegister08_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister09_cs = '1') then
                  eepDataRegister09_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister10_cs = '1') then
                  eepDataRegister10_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister11_cs = '1') then
                  eepDataRegister11_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister12_cs = '1') then
                  eepDataRegister12_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister13_cs = '1') then
                  eepDataRegister13_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister14_cs = '1') then
                  eepDataRegister14_reg <= eepDataRegister_io;
               end if;
               if (eepDataRegister15_cs = '1') then
                  eepDataRegister15_reg <= eepDataRegister_io;
               end if;               
            elsif (WbWriteEnable = '1') then
               if (mcbControl_cs = '1') then                      -- This 'compound register' shadows multiple Torrent registers to support
                  sequencerEnable_reg <= Data_in(0);              -- an identical register used in Monsoon Orange MCB and is required by software.
                  PauseExposure_reg <= Data_in(1);
                  SeqClkDiv_reg <= Data_in(3 downto 2);
                  DheIsSlave_reg <= not Data_in(4);
                  SyncDelay_reg <= Data_in(15 downto 8);                  
               end if;
               if (tempScanEnable_cs = '1') then
                  tempScanEnable_reg <= Data_in(0);
               end if;
               if (tempScanRate_cs = '1') then
                  tempScanRate_reg <= Data_in(9 downto 0);
               end if;
               if (eepDataRegister00_cs = '1') then
                  eepDataRegister00_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister01_cs = '1') then
                  eepDataRegister01_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister02_cs = '1') then
                  eepDataRegister02_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister03_cs = '1') then
                  eepDataRegister03_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister04_cs = '1') then
                  eepDataRegister04_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister05_cs = '1') then
                  eepDataRegister05_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister06_cs = '1') then
                  eepDataRegister06_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister07_cs = '1') then
                  eepDataRegister07_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister08_cs = '1') then
                  eepDataRegister08_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister09_cs = '1') then
                  eepDataRegister09_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister10_cs = '1') then
                  eepDataRegister10_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister11_cs = '1') then
                  eepDataRegister11_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister12_cs = '1') then
                  eepDataRegister12_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister13_cs = '1') then
                  eepDataRegister13_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister14_cs = '1') then
                  eepDataRegister14_reg <= Data_in(31 downto 0);
               end if;
               if (eepDataRegister15_cs = '1') then
                  eepDataRegister15_reg <= Data_in(31 downto 0);
               end if;
               if (shutterEnable_cs = '1') then
                  shutterEnable_reg <= Data_in(0);
               end if;
               if (shutterOpen_cs = '1') then
                  shutterOpen_reg <= Data_in(0);
               end if;
               if (preflashEnable_cs = '1') then
                  preflashEnable_reg <= Data_in(0);
               end if;
               if (preflashOn_cs = '1') then
                  preflashOn_reg <= Data_in(0);
               end if;
               if (shutterPol_cs = '1') then
                  shutterPol_reg <= Data_in(0);
               end if;
               if (preflashPol_cs = '1') then
                  preflashPol_reg <= Data_in(0);
               end if;
               if (shutterStatPol_cs = '1') then
                  shutterStatPol_reg <= Data_in(0);
               end if;
               if (shutterStatusForce_cs = '1') then
                  shutterStatusForce_reg <= Data_in(0);
               end if;
               if (sequencerEnable_cs = '1') then
                  sequencerEnable_reg <= Data_in(0);
               end if;
               if (DheIsSlave_cs = '1') then
                  DheIsSlave_reg <= Data_in(0);
               end if;
               if (IntegrationTime_cs = '1') then
                  IntegrationTime_reg <= Data_in;
               end if;
                if (PauseExposure_cs = '1') then
                  PauseExposure_reg <= Data_in(0);
               end if;
               if (loopReg00_cs = '1') then
                  loopDataReg00_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg01_cs = '1') then
                  loopDataReg01_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg02_cs = '1') then
                  loopDataReg02_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg03_cs = '1') then
                  loopDataReg03_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg04_cs = '1') then
                  loopDataReg04_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg05_cs = '1') then
                  loopDataReg05_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg06_cs = '1') then
                  loopDataReg06_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg07_cs = '1') then
                  loopDataReg07_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg08_cs = '1') then
                  loopDataReg08_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg09_cs = '1') then
                  loopDataReg09_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg10_cs = '1') then
                  loopDataReg10_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg11_cs = '1') then
                  loopDataReg11_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg12_cs = '1') then
                  loopDataReg12_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg13_cs = '1') then
                  loopDataReg13_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg14_cs = '1') then
                  loopDataReg14_reg <= Data_in(15 downto 0);
               end if;
               if (loopReg15_cs = '1') then
                  loopDataReg15_reg <= Data_in(15 downto 0);
               end if;               
               if (SyncDelay_cs = '1') then
                  SyncDelay_reg <= Data_in(7 downto 0);
               end if;               
               if (UserBits_cs = '1') then
                  UserBits_reg <= Data_in(3 downto 0);
               end if;               
               if (SeqClkDiv_cs = '1') then
                  SeqClkDiv_reg <= Data_in(1 downto 0);
               end if;               
            end if;
         end if;
      end process;

--------------------------------------------------------------------------------------------------
-- Write addressing error catcher
--------------------------------------------------------------------------------------------------
   AddrErr <= not (tempScanEnable_cs or tempScanRate_cs or eepRdCmdReg_cs or eepWrtCmdReg_cs or 
                  eepDataRegister00_cs or eepDataRegister01_cs or eepDataRegister02_cs or eepDataRegister03_cs or 
                  eepDataRegister04_cs or eepDataRegister05_cs or eepDataRegister06_cs or eepDataRegister07_cs or 
                  eepDataRegister08_cs or eepDataRegister09_cs or eepDataRegister10_cs or eepDataRegister11_cs or 
                  eepDataRegister12_cs or eepDataRegister13_cs or eepDataRegister14_cs or eepDataRegister15_cs or 
                  shutterEnable_cs or shutterOpen_cs or preflashEnable_cs or preflashOn_cs or shutterStatusForce_cs or
                  sequencerEnable_cs or DheIsSlave_cs or shutterPol_cs or preflashPol_reg or shutterStatPol_cs or
                  loopReg00_cs or loopReg01_cs or loopReg02_cs or loopReg03_cs or 
                  loopReg04_cs or loopReg05_cs or loopReg06_cs or loopReg07_cs or 
                  loopReg08_cs or loopReg09_cs or loopReg10_cs or loopReg11_cs or 
                  loopReg12_cs or loopReg13_cs or loopReg14_cs or loopReg15_cs or
                  SyncDelay_cs or UserBits_cs or PatMem_cs or PgmMem_cs or SeqClkDiv_cs or
                  IntegrationTime_cs or PauseExposure_cs or mcbControl_cs);

----------------------------------------------------------------------------------------------------
-- REGISTER READ LOGIC. Use the **_CS strobes to multiplex the registers (or module input ports)
-- to the data_out port.
----------------------------------------------------------------------------------------------------

-- Add future registers here as they are generated.
-- Although the hierarchy here has an inherent priority, there should only be one address CS strobe
-- true on any given cycle, therefore there is no priority as to where the registers should go in this list.

-- an compund register defined in the Monsoon Orange MCB and is required by software.
   mcbControl_reg <= SyncDelay_reg & "000" & (not DheIsSlave_reg) & SeqClkDiv_reg & PauseExposure_reg & sequencerEnable_reg;

   WbDataOut <=("0000000000000000000000000000000" & tempScanEnable_reg)     	when (tempScanEnable_cs = '1') else
               ("0000000000000000000000" & tempScanRate_reg)                	when (tempScanRate_cs = '1') else
               ("000000000000000000000000000" & DetectI2C_reg)              	when (DetectI2C_cs = '1') else
               (eepCmdStatus_i(3 downto 2) & "000000000000000000000000000000") when (eepRdCmdReg_cs = '1') else
               (eepCmdStatus_i(1 downto 0) & "000000000000000000000000000000") when (eepWrtCmdReg_cs = '1') else
					eepDataRegister00_reg    	                                    when (eepDataRegister00_cs = '1') else
					eepDataRegister01_reg    	                                    when (eepDataRegister01_cs = '1') else
					eepDataRegister02_reg    	                                    when (eepDataRegister02_cs = '1') else
					eepDataRegister03_reg    	                                    when (eepDataRegister03_cs = '1') else
					eepDataRegister04_reg    	                                    when (eepDataRegister04_cs = '1') else
					eepDataRegister05_reg    	                                    when (eepDataRegister05_cs = '1') else
					eepDataRegister06_reg    	                                    when (eepDataRegister06_cs = '1') else
					eepDataRegister07_reg    	                                    when (eepDataRegister07_cs = '1') else
					eepDataRegister08_reg    	                                    when (eepDataRegister08_cs = '1') else
					eepDataRegister09_reg    	                                    when (eepDataRegister09_cs = '1') else
					eepDataRegister10_reg    	                                    when (eepDataRegister10_cs = '1') else
					eepDataRegister11_reg    	                                    when (eepDataRegister11_cs = '1') else
					eepDataRegister12_reg    	                                    when (eepDataRegister12_cs = '1') else
					eepDataRegister13_reg    	                                    when (eepDataRegister13_cs = '1') else
					eepDataRegister14_reg    	                                    when (eepDataRegister14_cs = '1') else
					eepDataRegister15_reg    	                                    when (eepDataRegister15_cs = '1') else
					("00000000000000000000" & LCB_temperature0_i)               	when (LCB_temperature0_cs = '1') else
					("00000000000000000000" & LCB_temperature1_i)                  when (LCB_temperature1_cs = '1') else
					("00000000000000000000" & PSM_temperature0_i)                  when (PSM_temperature0_cs = '1') else
					("00000000000000000000" & PSM_temperature1_i)                  when (PSM_temperature1_cs = '1') else
					("00000000000000000000" & TSM_temperature0_i)                  when (TSM_temperature0_cs = '1') else
					("00000000000000000000" & TSM_temperature1_i)                  when (TSM_temperature1_cs = '1') else
					("00000000000000000000" & AFE0_temperature0_i)                 when (AFE0_temperature0_cs = '1') else
					("00000000000000000000" & AFE0_temperature1_i)                 when (AFE0_temperature1_cs = '1') else
					("00000000000000000000" & AFE1_temperature0_i)                 when (AFE1_temperature0_cs = '1') else
					("00000000000000000000" & AFE1_temperature1_i)                 when (AFE1_temperature1_cs = '1') else
               LCB_SSN_i                                                      when (LCB_SSN_cs = '1') else
               PSM_SSN_i                                                      when (PSM_SSN_cs = '1') else
               TSM_SSN_i                                                      when (TSM_SSN_cs = '1') else
               AFE0_SSN_i                                                     when (AFE0_SSN_cs = '1') else
               AFE1_SSN_i                                                     when (AFE1_SSN_cs = '1') else
               ("0000000000000000000000000000000" & shutterEnable_reg)        when (shutterEnable_cs = '1') else
               ("0000000000000000000000000000000" & shutterOpen_reg)          when (shutterOpen_cs = '1') else
               ("0000000000000000000000000000000" & preflashEnable_reg)       when (preflashEnable_cs = '1') else
               ("0000000000000000000000000000000" & preflashOn_reg)           when (preflashOn_cs = '1') else
               ("0000000000000000000000000000000" & shutterPol_reg)           when (shutterPol_cs = '1') else
               ("0000000000000000000000000000000" & preflashPol_reg)          when (preflashPol_cs = '1') else            
               ("0000000000000000000000000000000" & shutterStatPol_reg)       when (shutterStatPol_cs = '1') else            
               ("0000000000000000000000000000000" & shutterStatusForce_reg)   when (shutterStatusForce_cs = '1') else
               ("0000000000000000000000" & shutterOpenTime_i)                 when (shutterOpenTime_cs = '1') else
               ("0000000000000000000000" & shutterCloseTime_i)                when (shutterCloseTime_cs = '1') else
               ("000000000000000000000000000000" &  shutterStatus_i)          when (shutterStatus_cs = '1') else
               ("0000000000000000000000000000000" & sequencerEnable_reg)      when (sequencerEnable_cs = '1') else
               ("0000000000000000000000000000000" & DheIsSlave_reg)           when (DheIsSlave_cs = '1') else
               IntegrationTime_reg                                            when (IntegrationTime_cs = '1') else
               IntegrationCntr_i                                              when (IntegrationCntr_cs = '1') else
               ("0000000000000000000000000000000" & PauseExposure_reg)        when (PauseExposure_cs = '1') else
					("0000000000000000" & loopDataReg00_reg)    	                  when (loopReg00_cs = '1') else
					("0000000000000000" & loopDataReg01_reg)    	                  when (loopReg01_cs = '1') else
					("0000000000000000" & loopDataReg02_reg)    	                  when (loopReg02_cs = '1') else
					("0000000000000000" & loopDataReg03_reg)    	                  when (loopReg03_cs = '1') else
					("0000000000000000" & loopDataReg04_reg)    	                  when (loopReg04_cs = '1') else
					("0000000000000000" & loopDataReg05_reg)    	                  when (loopReg05_cs = '1') else
					("0000000000000000" & loopDataReg06_reg)    	                  when (loopReg06_cs = '1') else
					("0000000000000000" & loopDataReg07_reg)    	                  when (loopReg07_cs = '1') else
					("0000000000000000" & loopDataReg08_reg)    	                  when (loopReg08_cs = '1') else
					("0000000000000000" & loopDataReg09_reg)    	                  when (loopReg09_cs = '1') else
					("0000000000000000" & loopDataReg10_reg)    	                  when (loopReg10_cs = '1') else
					("0000000000000000" & loopDataReg11_reg)    	                  when (loopReg11_cs = '1') else
					("0000000000000000" & loopDataReg12_reg)    	                  when (loopReg12_cs = '1') else
					("0000000000000000" & loopDataReg13_reg)    	                  when (loopReg13_cs = '1') else
					("0000000000000000" & loopDataReg14_reg)    	                  when (loopReg14_cs = '1') else
					("0000000000000000" & loopDataReg15_reg)    	                  when (loopReg15_cs = '1') else               
					("000000000000000000000000" & SyncDelay_reg)    	            when (SyncDelay_cs = '1') else
					("0000000000000000000000000000" & UserBits_reg)    	         when (UserBits_cs = '1') else
					("000000000000000000000000000000" & SeqClkDiv_reg)    	      when (SeqClkDiv_cs = '1') else
 					("0000000000000000" & PatternMemData_i)    	                  when (PatMem_cs = '1') else               
					("0000000000000000" & ProgramMemData_i)    	                  when (PgmMem_cs = '1') else 
					("0000000000000000" & SeqCmdReg_i)    	                        when (SeqCmdReg_cs = '1') else 
					SeqStatus_i                         	                        when (SeqStatus_cs = '1') else
               ("000000000000000000000000" & SeqEnablesReg_i)                 when (SeqEnables_cs = '1') else
               SeqTestCondReg_i                                               when (SeqTestReg_cs = '1') else
               ("0000000000000000" & mcbControl_reg)                          when (mcbControl_cs = '1') else 
               WbStatIn              								                  when (CfgServModInStatus_cs = '1') else
               LocalStatus_reg              								            when (CfgServModOutStatus_cs = '1') else
               ("0000000000000000" & MODULE_IDENT)					            	when (ModuleId_cs = '1') else
               ("0000000000000000" & CODE_IDENT)              						when (CodeId_cs = '1') else
               x"42424242"; --- Code word for bad bus read address :-)

end Behavioral;

