------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   GENERIC
-- DEVICE ID    :   U22 (IR ACQ FPGA)
-- PARENT MODULE:   SEQUENCER TOP LEVEL (SEQ_FPGA)  
-- MODULE       :   Clock Divider 
-- VERSION      :   4.2 
------------------------------------------------------------------------
-- FILE NAME    :   Clock_Divide.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   05/14/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- The module based upon the select signals divides the system clock 
-- 00 SeqClk : SYSCLK / 1 = 20 MHz 
-- 01 SeqClk : SYSCLK / 2 = 10 MHz
-- 10 SeqClk : SYSCLK / 4 =  5 MHz
-- 11 SeqClk : SYSCLK / 8 =  2.5 MHz
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	      AUTHOR  VERSION  MODIFICATIONS
-- 03/12/2003  KAVI     1.0     ORIGINAL VERSION
-- 07/11/2003   KC		2.0		Changed the Implementation to gated clock in 
--                              order to minimize clock skew
-- 05/14/2004  PCM      4.2     ADDED CLOCK SYNCHRONIZATION, CLEANED UP THE CODE.		
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- CLKIN	: System Clock 40 MHz.
-- RATIO    : Clock Divde input select signals 
-- CLKOUT   : Divided sequencer clock output
-- SYNC     : rising edge syncs the clocks.
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Clock_Divide is
   Port (CLKIN    : in  std_logic;
         RATIO    : in  std_logic_vector(1 downto 0);
         CLKOUT   : out std_logic;
		   SYNC     : in  std_logic
      );
end Clock_Divide;

architecture Behavioral of Clock_Divide is
	signal DV : std_logic_vector(3 downto 0);
   signal SyncLock : std_logic;
   signal SyncClk  : std_logic;
begin

--	A clock divider with asynchronous clear 
	ClkDivideProc : process (CLKIN, SyncClk)
	begin
      if (SyncClk = '1') then
            DV <= (others => '1');
            CLKOUT <= '0';    
      elsif (rising_edge (CLKIN)) then
         case RATIO is
            when "00" =>
               CLKOUT <= DV(0);
            when "01" =>
               CLKOUT <= DV(1);
            when "10" =>
               CLKOUT <= DV(2);
            when "11" =>
               CLKOUT <= DV(3);
            when others =>
               null;
         end case;      
			DV <= DV + 1;
		end if;
	end process;

    ClkSyncProc : process (CLKIN)
    begin
      if (rising_edge(CLKIN)) then
         SyncLock <= SYNC;
      end if;
    end process;

   SyncClk <= (SYNC xor SyncLock) and SYNC;         -- Detect the rising edge

end Behavioral;
