------------------------------------------------------------------------
-- Copyright Notice:
-- (c) Copyright 2004 - 2008, National Optical Astronomy Observatory,
-- which is operated for the National Science Foundation by the
-- Association of Universities for Research in Astronomy under
-- Cooperative Agreement No. AST-0132798.
------------------------------------------------------------------------
-- PROJECT      :   MONSOON-TORRENT
-- BOARD        :   LOCAL CONTROL BOARD
-- DEVICE ID    :   VIRTEX 5LX
-- PARENT MODULE:   CFG_SERVICES
-- MODULE       :   SHUTTER CONTROL
-- VERSION      :  2.10
------------------------------------------------------------------------
-- FILE NAME    :   ShutterControl_Ver210.vhd
-- AUTHOR         :   Peter Moore
-- DATE               :   10/02/2011
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 10.1.03
------------------------------------------------------------------------
-- DESCRIPTION  :  Controls the shutter and preflash control and timing functions
--
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- Shutter status values - 00 = shutter moving, 01 = shutter open, 10 = shutter closed, 11 = shutter error. 
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	     AUTHOR  VERSION  MODIFICATIONS
-- 06/22/2009  PCM     1.00    INITIAL VERSION
-- 01/26/2010    PCM     1.01    Modified for sequencer insertion. Removed the integration timer to the sequencer module.
--  01/19/2011    PCM     2.01    Added polarity select capability
-- 10/02/2011    PCM     2.10    Changed timer clock to external source. Debugged shutter timers, made SeqShutStatus_o do tricks for dark integrations, etc. so that integration timer will run. 
--                                                 Added shutter status polarity control, Added shutter status debounce logic.
------------------------------------------------------------------------
-- PORT DEFINITIONS :
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ShutterControl_Ver210 is
	Port (
-- Control signals
		SysClk_i 				: in  std_logic;
      OneMsClk_i           : in  std_logic;
		Reset_i 					: in  std_logic;
      ShutterEnable_i      : in  std_logic;  -- Enables shutter operation controlled by the sequencer
      ShutterOpen_i        : in  std_logic;  -- Force shutter open command
      PreflashEnable_i     : in  std_logic;  -- Enables preflash operation by the sequencer
      PreflashOn_i         : in  std_logic;  -- Force preflash on command
      ShutterStatusForce_i : in  std_logic;  -- Force overide the of the shutter status input to show status according to commanded shutter state
-- Sequencer control input
      SeqShutterOpen_i     : in  std_logic;
      SeqPreflashOn_i      : in  std_logic;
      shutterPolarity_i    : in  std_logic;  -- sets the output polarity of the shutter drive signal
      preflashPolarity_i   : in  std_logic;  -- sets the output polarity of the preflash signal
      shutterStatPolarity_i: in  std_logic;  -- sets the output polarity of the two shutter status signals - 0=low true status signal (opto-isolator not turned on); 1=High true status signal (opto-isolator turned on).      
-- Shutter status signals
      shutterOpenTime_o    : out std_logic_vector(9 downto 0); -- Time in milliseconds from shutter open command to receiving shutter open status
      shutterCloseTime_o   : out std_logic_vector(9 downto 0); -- Time in milliseconds from shutter close command to receiving shutter close status
      shutterStatus_o      : out std_logic_vector(1 downto 0); -- Shutter status input from hardware - 00 = forced status, 01 = shutter open, 10 = shutter closed, 11 = shutter error.
-- Hardware signals
      PreflashOn_o         : out std_logic;
      ShutterOpen_o        : out std_logic;
      ShutterOpenStatus_i  : in  std_logic;
      ShutterCloseStatus_i : in  std_logic      
   );
end ShutterControl_Ver210;

architecture Behavioral of ShutterControl_Ver210 is

-- Shutter debounce signals
   signal ShutterCloseStatus, ShutterOpenStatus : std_logic;
   signal StOpen, StClose : std_logic;
   signal StOpenCntr, StCloseCntr : std_logic_vector(7 downto 0);
   signal DbClk : std_logic;
   signal DbClkCntr : std_logic_vector(6 downto 0);
   constant DBCLK_COUNT : std_logic_vector(6 downto 0) := "1100110";  -- 103 counts of sysClk to give approx. 0.5millisec / 256 timebase.
--   constant DBCLK_COUNT : std_logic_vector(6 downto 0) := "0000001";  -- 103 counts of sysClk to give approx. 0.5millisec / 256 timebase.
   
-- Shutter control signals
   signal OpenShutter, OpenPreflash, OpenCntrEnable, OpenFlag, CloseCntrEnable, CloseFlag : std_logic;
   signal ShutterOpenCntr, ShutterCloseCntr : std_logic_vector(9 downto 0);
   signal ShutterStatus : std_logic_vector(1 downto 0);
   signal SClosed, SOpen, SMoving, SFail : std_logic;

begin
-- Shutter control logic
   ShutterOpen_o <= OpenShutter xor shutterPolarity_i;    -- take into account the output polarity required 
   OpenShutter   <= shutterOpen_i or (SeqShutterOpen_i and shutterEnable_i);
   
   PreflashOn_o  <= OpenPreflash xor preflashPolarity_i;    -- take into account the output polarity required
   OpenPreflash  <= preflashOn_i or (SeqPreflashOn_i and preflashEnable_i);
   
   shutterStatus_o <= ShutterStatus; 
   shutterOpenTime_o  <= ShutterOpenCntr;
   shutterCloseTime_o <= ShutterCloseCntr;

-- Shutter status debounce circuits
-- Normalize the status signal polarity
   StClose <= ShutterCloseStatus_i xor shutterStatPolarity_i;
   StOpen   <= ShutterOpenStatus_i xor shutterStatPolarity_i;

-- Establish a time base to samplethe status signals = approx. 512KHz
   DbClkGenProc : process(SysClk_i, Reset_i)
   begin
      if (rising_edge(SysClk_i)) then
         if (Reset_i = '1') then
            DbClk <= '0';
            DbClkCntr <= (others => '0');
         elsif (DbClkCntr >= DBCLK_COUNT) then
            DbClk <= '1';
            DbClkCntr <= (others => '0');
         else
            DbClk <= '0';
            DbClkCntr <= DbClkCntr + 1;
         end if;
      end if;
   end process;

-- Asymetric schmidt trigger for the open status. Status goes false on first occurance of false, status goes true after 256 samples prove status true
   OpenStatusDebounceProc : process (DbClk, Reset_i, StOpen)
   begin
      if (Reset_i = '1') then
         StOpenCntr <= (others => StOpen);
      elsif (rising_edge(DbClk)) then
         if (StOpen = '1' and ShutterOpenStatus = '0') then
            StOpenCntr <= StOpenCntr + 1;
         else
            StOpenCntr <= (others => StOpen);
         end if;
      end if;
   end process;
   
   ShutterOpenStatus <= StOpenCntr(0) and StOpenCntr(1) and StOpenCntr(2) and StOpenCntr(3) and StOpenCntr(4) and StOpenCntr(5) and StOpenCntr(6) and StOpenCntr(7);

-- Asymetric schmidt trigger for the close status.    
   CloseStatusDebounceProc : process (DbClk, Reset_i, StClose)
   begin
      if (Reset_i = '1') then
         StCloseCntr <= (others => StClose);
      elsif (rising_edge(DbClk)) then
         if (StClose = '1' and ShutterCloseStatus = '0') then
            StCloseCntr <= StCloseCntr + 1;
         else
            StCloseCntr <= (others => StClose);
         end if;
      end if;
   end process;
   
   ShutterCloseStatus <= StCloseCntr(0) and StCloseCntr(1) and StCloseCntr(2) and StCloseCntr(3) and StCloseCntr(4) and StCloseCntr(5) and StCloseCntr(6) and StCloseCntr(7);
   
   
-- Latch Shutter status values from mux - 00 = shutter moving, 01 = shutter open, 10 = shutter closed, 11 = shutter error. 
   ShutterStatusLatchProc : process(SysClk_i)
   begin
      if (rising_edge(SysClk_i)) then
         if (shutterStatusForce_i = '0') then
            ShutterStatus(1) <= ShutterCloseStatus;
            shutterStatus(0) <= ShutterOpenStatus;
         else
            ShutterStatus(1) <= not OpenShutter;
            ShutterStatus(0) <= OpenShutter;
         end if;
      end if;
   end process;

-- decoded states for shutter status
   SClosed <= ShutterStatus(1) and not ShutterStatus(0);
   SOpen   <= ShutterStatus(0) and not ShutterStatus(1);
   SMoving <= not (ShutterStatus(0) or ShutterStatus(1));
   SFail   <= ShutterStatus(0) and ShutterStatus(1);
   
-- Shutter timer logic
-- Counts milliseconds while shutter is transitioning from closed to open
   OpenShutterCntrProc : process (SysClk_i)
   begin
      if (rising_edge(SysClk_i)) then
         if (SClosed = '1' or SFail = '1') then
            OpenCntrEnable <= '1';
            OpenFlag <= '1';
         elsif (SMoving = '1' and OpenCntrEnable = '1') then
            if (OpenFlag = '1') then
               ShutterOpenCntr <= (others => '0');
               OpenFlag <= '0';
            elsif (OneMsClk_i = '1') then
               ShutterOpenCntr <= ShutterOpenCntr + 1;
            end if;
         else
            OpenCntrEnable <= '0';
         end if;
      end if;
   end process;
   
-- Counts milliseconds while shutter is transitioning from open to closed
   CloseShutterCntrProc : process (SysClk_i)
   begin
      if (rising_edge(SysClk_i)) then
         if (SOpen = '1' or SFail = '1') then
            CloseCntrEnable <= '1';
            CloseFlag <= '1';
         elsif (SMoving = '1' and CloseCntrEnable = '1') then
            if (CloseFlag = '1') then
               ShutterCloseCntr <= (others => '0');
               CloseFlag <= '0';
            elsif (OneMsClk_i = '1') then
               ShutterCloseCntr <= ShutterCloseCntr + 1;
            end if;
         else
            CloseCntrEnable <= '0';
         end if;
      end if;
   end process;
   
end Behavioral;
