------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON-TORRENT
-- BOARD        :   LOCAL CONTROL BOARD
-- DEVICE ID    :   VIRTEX 5LX
-- PARENT MODULE:   CFG_SERVICES
-- MODULE       :   DHE SYNCHRONIZATION LOGIC
-- VERSION      :   1.00
------------------------------------------------------------------------
-- FILE NAME    :   SYNCLOGIC_VER101.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   01/26/2010
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 10.1.03
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- PROVIDES THE TIMING TO ENSURE THAT MULTIPLE DHE UNITS ARE SYNCHRONIZED
-- IN THE EXECUTION OF THE SEQUENCER MPU PROGRAM. 
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- This is a non-retriggerable one shot that provides the following waveform
-- (Several events occur here) :
--                     /-----------\          /--\
-- SYNC_EFR     ______/             \________/    \___
--                                  |
--                      /-----------|---------------------------\
-- EXT_SYNC_OUT _______/            |                            \_________
--                     |            |------------ DELAY_MS ------|
--                     |                                         |
--                     |<= Seq clock sync'ed on rising edge      |
--                                                               |
--           Typical [#WAIT JCB #SYNC #WAIT] instruction falls   |
--           out of loop on trailing edge   ====================>|
--
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	     AUTHOR  VERSION  MODIFICATIONS
-- 01/27/2010   PCM     1.01  Adapted logic from Orange version 4.3 ( Seq_Mpuv465) for use with Torrent Register structure.
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity SyncLogic_Ver101 is
   Port (SeqClk       : in  std_logic;    -- Sequencer clock
         SEQ_RUNNING  : in  std_logic;    -- Resets the logic to known state when not true.
         DHE_IS_MASTER: in  std_logic;    -- flag to indicate whether we are master or slave dhe.
         SYNC_DELAY   : in  std_logic_vector(7 downto 0);   -- Delay (in ms.) from when SYNC_EFR changes state to when EXT_SYNC_OUT goes true.
         EXT_SYNC     : inout std_logic;  -- SYNC bit input from previous dhe in daisy chain or SYNC bit out to next DHE depending on DHE_IS_MASTER signal
         SYNC_EFR     : in  std_logic;    -- SYNC request from our sequencer if we are master
         SYNC_FLAG    : out std_logic     -- flag that indicates sync status to the sequencer
         );
end SyncLogic_Ver101;

architecture Behavioral of SyncLogic_Ver101 is
    constant ONE_MILLISEC : std_logic_vector(15 downto 0) := x"9C40";
    signal MSecCntr : std_logic_vector(15 downto 0);
    signal DelayCnt : std_logic_vector(7 downto 0);
    signal DlyCntr  : std_logic_vector(7 downto 0);
    signal MSecFlag : std_logic;
    signal Trigger  : std_logic;
    signal TrigArm  : std_logic;
    signal SyncBit  : std_logic;
begin

    EXT_SYNC     <= (SyncBit or TrigArm) when (DHE_IS_MASTER = '1') else 'Z';

------------------------------------------------------------------------
-- NEGATIVE EDGE TRIGGERED ONE SHOT
------------------------------------------------------------------------
    NegEdgeTrigProc : process(SeqClk)
    begin
        if (rising_edge(SeqClk)) then
            if (SEQ_RUNNING = '1' and SyncBit = '0') then
                if (SYNC_EFR = '1') then
                    TrigArm <= '1';
                elsif (TrigArm = '1') then
                    Trigger <= '1';
                    DelayCnt <= SYNC_DELAY;           -- Capture the required delay time 
                end if;
            else
                TrigArm <= '0';
                Trigger <= '0';
            end if;
        end if;
    end process;

------------------------------------------------------------------------
-- DELAY TIMER
------------------------------------------------------------------------
    DlySyncProc : process(SeqClk)
    begin
        if (rising_edge(SeqClk)) then
            if (SEQ_RUNNING = '1') then
                if (DlyCntr < DelayCnt) then
                    if (MSecFlag = '1') then
                        DlyCntr <= DlyCntr + 1;
                    end if;
                elsif (SyncBit = '1') then
                    SyncBit <= '0';
                elsif (Trigger = '1') then
                    DlyCntr <= (others => '0');
                    SyncBit <= '1';
                end if;
            else
                DlyCntr <= (others => '1');
                SyncBit <= '0';
            end if;
        end if;
    end process;

------------------------------------------------------------------------
-- MILLISECOND TIMER
------------------------------------------------------------------------
    MilliSecTimerProc : process (SeqClk)
    begin
        if (rising_edge(SeqClk)) then
            if (TrigArm = '1') then             -- Hold off to synchronize timer to edge.
                MSecCntr <= (others => '0');
                MSecFlag <= '0';
            elsif (MSecCntr < ONE_MILLISEC) then
                MSecCntr <= MSecCntr + 1;
                MSecFlag <= '0';
            else
                MSecCntr <= (others => '0');
                MSecFlag <= '1';
            end if;
        end if;
    end process;

------------------------------------------------------------------------
-- SYNC IN CONDITIONING
------------------------------------------------------------------------

	SyncInDetectProcess : process (SeqClk)
	begin
		if (rising_edge(SeqClk)) then
			SYNC_FLAG <= EXT_SYNC;
		end if;
	end process; 

end Behavioral;
