-----------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON-TORRENT
-- BOARD        :   LOCAL CONTROL BOARD
-- DEVICE ID    :   VIRTEX 5LX
-- PARENT MODULE:   CFG_SERVICES
-- MODULE       :   TEST CONDITION REGISTER
-- VERSION      :   1.03
------------------------------------------------------------------------
-- FILE NAME    :   TestCondReg_Ver103.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   03/15/2010
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 10.1.03
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- WRITING TO THIS REGISTER INITIATES A BUS READ WHRE THE RESULT OF THE READ IS AVAILABLETO
-- THE SEQUENCER TO PERFORM BIT TESTS AND CONDITIONAL JUMPS. tHE WRITE TO THIS REGISTER IS
-- BY ADDRESSING LOCATION 0x0109 WITH THE CFG BOARD SELECT TRUE. DATA FROM THE PATTERN MEMORY
-- IS USED TO FORM THE BOARD SELECT AND ON BOARD ADDRESS OF THE REGISTER TO READ.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- PATTERN MEMORY BITS AND THEIR FUNCTION
-- BITS								BIT
-- 31 DOWNTO 14 : NOT USED
-- 23 DOWNTO 16 : BOARD SELECT VALUE
-- 15 DOWNTO 0	 : BOARD ADDRESS
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	        AUTHOR  VERSION  MODIFICATIONS
-- 03/15/2010   PCM     1.03      INITIAL RELEASE
--
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TestCondReg_Ver103 is
    Port (SeqClk_i	 		: in  std_logic;                       -- Sequencer clock - can be SSysClk / 2, SysClk / 4, or SysClk / 8
         SysClk_i          : in  std_logic;                       -- 53.125 MHz system clock for bus transactions
         SeqRunFlag_i      : in  std_logic;                       -- True when sequencer is actually running code
         WrtEnable_i 	   : in  std_logic;                       -- STROBE TRUE WHEN MODULE SELECT = 4 AND MODE = WRT32 AND LDA REGISTER = 0x0109
         PatternData_i     : in  std_logic_vector(23 downto 0);   -- Input to determine module select and address values
         WbRdReq_o         : out std_logic;                       -- Read request to Wishbone master interface
         WbRdAck_i         : in  std_logic;                       -- Read acknowledge from Wishbone master
         WbRdAddr_o        : out std_logic_vector(23 downto 0);   -- mmodule select and Address output to Wishbone master
         WbRdData_i        : in  std_logic_vector(31 downto 0);   -- Read data from module that was read to be saved in the test condition register
         TestCondReg_o     : out std_logic_vector(31 downto 0)    -- Test condition regsietr available to sequencer
         );
end TestCondReg_Ver103;

architecture Behavioral of TestCondReg_Ver103 is

   signal ReqAddress : std_logic_vector(23 downto 0);
   signal TriggerFlag, TriggerAck : std_logic;
   signal TestCondReg : std_logic_vector(31 downto 0);

begin

-- Output data 
   TestCondReg_o <= TestCondReg;

-- Capture the sequencer command
   AddresscaptureProc : process(SeqClk_i)
   begin
      if (rising_edge(SeqClk_i)) then
         if (WrtEnable_i = '1') then
            ReqAddress <= PatternData_i;
         end if;
      end if;
   end process;

   TriggercaptureProc : process(SeqRunFlag_i, TriggerAck, SeqClk_i)
   begin
      if (TriggerAck = '1' or SeqRunFlag_i = '0') then
         TriggerFlag <= '0';
      elsif (rising_edge(SeqClk_i) and TriggerFlag = '0') then
         TriggerFlag <= WrtEnable_i;
      end if;
   end process;

-- Process the read request
   WbRequestProc : process(SysClk_i)
   begin
      if (rising_edge(SysClk_i)) then
         if (SeqRunFlag_i = '0') then
            WbRdReq_o <= '0';
            TriggerAck  <= '0';
         elsif (TriggerFlag = '1' and TriggerAck = '0') then
            WbRdReq_o <= '1';
            WbRdAddr_o <= ReqAddress;
            TriggerAck  <= '1';
         elsif (WbRdAck_i = '1') then
            TestCondReg <= WbRdData_i;
            TriggerAck  <= '0';
         else
            WbRdReq_o <= '0';
         end if;
      end if;
   end process;

end Behavioral;


